package net.sourceforge.noadsunpacker.gui;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextPane;

/**
 * This is a dialog to report errors. It shows an error message,
 * an extended error information (e.g. exception stack trace),
 * a flag (optionally) and a button "OK".
 *
 * @author Fedor Uvarov
 */
public class ErrorReportingDialog extends JDialog {
public static final String OKBUTTON_TEXT = "OK";
private JLabel UsedLabel;
private JTextPane UsedTextPane;
private JButton OKButton;
private static ErrorReportingDialog CurrentlyShownFrame = null;
private static final Object lockHolder = new Object();

private ErrorReportingDialog(Frame owner) {
    super(owner);

    this.setResizable(false);
    this.setUndecorated(true);
    this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.weightx = 1;
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.insets = new Insets(5, 5, 10, 5);
    gbc.fill = GridBagConstraints.HORIZONTAL;
    this.setLayout(gbl);
    this.setPreferredSize(new Dimension(600, 700));

    UsedLabel = new JLabel();
    UsedLabel.setHorizontalAlignment(JLabel.CENTER);
    this.add(UsedLabel);
    gbc.weighty = 0.3;
    gbl.setConstraints(UsedLabel, gbc);
    gbc.gridy++;

    UsedTextPane = new JTextPane();
    this.add(UsedTextPane);
    gbc.weighty = 0.6;
    gbl.setConstraints(UsedTextPane, gbc);
    gbc.gridy++;

    JPanel p = new JPanel();
    p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
    p.add(Box.createHorizontalGlue());
    OKButton = new JButton(OKBUTTON_TEXT);
    OKButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            CurrentlyShownFrame.setVisible(false);
        }
    });
    p.add(OKButton);
    this.add(p);
    gbc.weighty = 0.1;
    gbc.insets = new Insets(0, 5, 5, 5);
    gbl.setConstraints(p, gbc);
}

private void prepare(String Caption, String ErrorText, String ExtendedErrorText) {
    this.pack();
    UsedTextPane.setMaximumSize(UsedTextPane.getSize());
    this.setTitle(Caption);
    UsedLabel.setText(ErrorText);
    UsedTextPane.setText(ExtendedErrorText);
    this.setLocationRelativeTo(null);
}

private void printException(Throwable e) {
    StringBuilder str = new StringBuilder(e.toString()+"\n");
    for(StackTraceElement el : e.getStackTrace()) {
        str.append("   at "+el.toString()+"\n");
    }
    UsedTextPane.setText(str.toString());
}

public static void showErrorMessage(Frame owner, String Caption, String ErrorText,
        String ExtendedErrorText) {
    if(CurrentlyShownFrame == null || CurrentlyShownFrame.getOwner() != owner)CurrentlyShownFrame = new ErrorReportingDialog(owner);
    CurrentlyShownFrame.prepare(Caption, ErrorText, ExtendedErrorText);
    CurrentlyShownFrame.setVisible(true);
}

public static void showErrorMessage(Frame owner, String Caption, String ErrorText,
        Exception ReportedException) {
    if(CurrentlyShownFrame == null || CurrentlyShownFrame.getOwner() != owner)CurrentlyShownFrame = new ErrorReportingDialog(owner);
    CurrentlyShownFrame.prepare(Caption, ErrorText, "");
    CurrentlyShownFrame.printException(ReportedException);
    CurrentlyShownFrame.setVisible(true);
}
}
