package net.sourceforge.noadsunpacker.gui;

import com.l2fprod.common.swing.JDirectoryChooser;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.filechooser.FileNameExtensionFilter;
import net.sourceforge.noadsunpacker.core.ArchiveUnpackManager;
import net.sourceforge.noadsunpacker.core.ArchiveUnpackManagerException;
import net.sourceforge.noadsunpacker.core.ArchiveUnpackManagerReportReceiver;
import net.sourceforge.noadsunpacker.core.UnpackException;

/**
 * The main GUI window
 *
 * @author Fedor Uvarov
 */
public class MainFrame extends JFrame {
public static final String GREETING_EN =
        "<html><center><b style=\"font-size:18;\">NoAds FileUnpacker</b><br>" +
        "<b style=\"font-size:14;\">0.1.1</b></center>" +
        "<br>" +
        "To unpack an archive, choose the archive file (.tmfp or .exe)," +
        "<br>select a directory to unpack the file to, and click \"Unpack\".</html>";
public static final int BOTTOM_BUTTONS_STRUT_SIZE = 5;
private JTextField FileNameField;
private JTextField DestDirField;
private JFileChooser UsedFileChooser;
private JDirectoryChooser UsedDirectoryChooser;
private JComponent[] ComponentsToDisable;
private UnpackProgressDialog UsedUPD;
private final MainFrame ThisFrame = this;
private ArchiveUnpackManager UsedArchiveUnpackManager;
private HelpFrame UsedHelpFrame = null;

public MainFrame(String Source, String Target) {
    this.setTitle("NoAds FileUnpacker");

    //Preparing an array for components that are to be disabled while unpacking
    ComponentsToDisable = new JComponent[7];
    int CTDCounter = 0;

    //Preparing file and directory choosers
    UsedFileChooser = new JFileChooser();
    UsedFileChooser.setAcceptAllFileFilterUsed(false);  //I don't want it to be the first filter in the list
    FileNameExtensionFilter stdfilter = new FileNameExtensionFilter("TMFP and self-extracting TMFP archives (*.tmfp, *.exe)", "tmfp", "exe");
    UsedFileChooser.addChoosableFileFilter(stdfilter);
    UsedFileChooser.addChoosableFileFilter(new FileNameExtensionFilter("TMFP archives (*.tmfp)", "tmfp"));
    UsedFileChooser.addChoosableFileFilter(new FileNameExtensionFilter("Self-extracting archives (*.exe)", "exe"));
    UsedFileChooser.addChoosableFileFilter(UsedFileChooser.getAcceptAllFileFilter());
    UsedFileChooser.setFileFilter(stdfilter);
    UsedFileChooser.setFileHidingEnabled(false);

    UsedDirectoryChooser = new JDirectoryChooser();
    UsedDirectoryChooser.setName("Choose directory");

    //GridBagConstraints for the form itself
    GridBagLayout MainFrameGBL = new GridBagLayout();
    GridBagConstraints MainGBC = new GridBagConstraints();
    this.setLayout(MainFrameGBL);

    JLabel l = new JLabel(GREETING_EN);
    this.add(l);
    MainGBC.gridx=0;
    MainGBC.gridy=0;
    MainGBC.fill = GridBagConstraints.HORIZONTAL;
    MainGBC.insets = new Insets(5, 5, 10, 5);
    MainFrameGBL.setConstraints(l, MainGBC);

    //GBC's for selection panels
    GridBagConstraints DescrLabelGBC = new GridBagConstraints();
    DescrLabelGBC.gridx = 0;
    DescrLabelGBC.gridy = 0;
    DescrLabelGBC.weightx = 1;
    DescrLabelGBC.anchor = GridBagConstraints.LINE_START;
    GridBagConstraints TextFieldGBC = new GridBagConstraints();
    TextFieldGBC.gridx = 0;
    TextFieldGBC.gridy = 1;
    TextFieldGBC.weightx = 1;
    TextFieldGBC.fill = GridBagConstraints.HORIZONTAL;
    GridBagConstraints BrowseButtonGBC = new GridBagConstraints();
    BrowseButtonGBC.gridx = 1;
    BrowseButtonGBC.gridy = 1;
    BrowseButtonGBC.anchor = GridBagConstraints.LINE_END;

    //Adding "file selection panel"
    JPanel p = new JPanel();
    GridBagLayout gbl = new GridBagLayout();
    p.setLayout(gbl);
    l = new JLabel("File to unpack: ");
    p.add(l);
    gbl.setConstraints(l, DescrLabelGBC);
    FileNameField = new JTextField(Source);
    ComponentsToDisable[CTDCounter] = FileNameField;
    CTDCounter++;
    gbl.setConstraints(FileNameField, TextFieldGBC);
    p.add(FileNameField);
    JButton b = new JButton("...");
    ComponentsToDisable[CTDCounter] = b;
    CTDCounter++;
    b.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            showOpenFileDialog();
        }
    });
    //b.setMaximumSize(new Dimension(b.getWidth(), FileNameField.getHeight()));
    p.add(b);
    gbl.setConstraints(b, BrowseButtonGBC);
    this.add(p);
    MainGBC.gridy++;
    MainFrameGBL.setConstraints(p, MainGBC);

    p = new JPanel();
    gbl = new GridBagLayout();
    p.setLayout(gbl);
    l = new JLabel("Destination directory: ");
    p.add(l);
    gbl.setConstraints(l, DescrLabelGBC);
    DestDirField = new JTextField(Target);
    ComponentsToDisable[CTDCounter] = DestDirField;
    CTDCounter++;
    p.add(DestDirField);
    gbl.setConstraints(DestDirField, TextFieldGBC);
    b = new JButton("...");
    ComponentsToDisable[CTDCounter] = b;
    CTDCounter++;
    b.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            showChooseDirectoryDialog();
        }
    });
    //b.setMaximumSize(new Dimension(b.getWidth(), DestDirField.getHeight()));
    p.add(b);
    gbl.setConstraints(b, BrowseButtonGBC);
    this.add(p);
    MainGBC.gridy++;
    MainFrameGBL.setConstraints(p, MainGBC);

    //Adding buttons panel
    p = new JPanel();
    p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
    b = new JButton("Help");
    ComponentsToDisable[CTDCounter] = b;
    CTDCounter++;
    b.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            if(UsedHelpFrame != null)
                UsedHelpFrame.setVisible(true);
        }
    });
    p.add(b);
    /*p.add(Box.createHorizontalStrut(BOTTOM_BUTTONS_STRUT_SIZE));
    b = new JButton("About");
    ComponentsToDisable[CTDCounter] = b;
    CTDCounter++;
    b.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            //TODO: Open about box
        }
    });
    p.add(b);*/
    p.add(Box.createHorizontalGlue());
    b = new JButton("Exit");
    ComponentsToDisable[CTDCounter] = b;
    CTDCounter++;
    b.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            closeFrame();
        }
    });
    p.add(b);
    p.add(Box.createHorizontalStrut(BOTTOM_BUTTONS_STRUT_SIZE));
    b = new JButton("Unpack");
    ComponentsToDisable[CTDCounter] = b;
    CTDCounter++;
    b.setFont(b.getFont().deriveFont((int)Font.BOLD));
    b.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            startUnpacking();
        }
    });
    p.add(b);
    this.add(p);
    MainGBC.gridy++;
    MainGBC.insets = new Insets(5, 5, 5, 5);
    MainFrameGBL.setConstraints(p, MainGBC);

    //Creating the unpack progress dialog
    UsedUPD = new UnpackProgressDialog(this);

    //Initializing the help frame
    try {
        UsedHelpFrame = new HelpFrame();
    }
    catch(IOException e) {
        ErrorReportingDialog.showErrorMessage(ThisFrame, "Error while opening help",
                "An error has occured while constructing help window. More details below.", e);
    }

    this.setDefaultCloseOperation(EXIT_ON_CLOSE);
    this.setResizable(false);
    this.pack();
    this.setLocationRelativeTo(null);
}

private void showOpenFileDialog() {
    if(UsedFileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
        try {
            FileNameField.setText(UsedFileChooser.getSelectedFile().getCanonicalPath());
        }
        catch(IOException e) {
            JOptionPane.showMessageDialog(this, "An error has occured", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }
}

private void showChooseDirectoryDialog() {
    if(UsedDirectoryChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
        try {
            DestDirField.setText(UsedDirectoryChooser.getSelectedFile().getCanonicalPath());
        }
        catch(IOException e) {
            JOptionPane.showMessageDialog(this, "An error has occured", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }
}

private void closeFrame() {
    if(UsedHelpFrame != null) UsedHelpFrame.dispose();
    this.dispose();
}

public static void main(String[] args) {
    MainFrame x = new MainFrame("", "");
    x.setVisible(true);
}

private void startUnpacking() {
    if(FileNameField.getText().trim().isEmpty() || DestDirField.getText().trim().isEmpty())
        JOptionPane.showMessageDialog(this, "Please specify the target file and destination directory", "Error", JOptionPane.WARNING_MESSAGE);
    else {
        File dest = new File(DestDirField.getText());
        if(!dest.exists()) {
            int i = JOptionPane.showConfirmDialog(this, "The destination directory does not exist. Create it?", "Creating directory", JOptionPane.YES_NO_OPTION);
            if(i == JOptionPane.YES_OPTION) {
                if(!dest.mkdirs()) {
                    JOptionPane.showMessageDialog(this, "Could not create the destination directory", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }
            }
        }
        for(JComponent c : ComponentsToDisable)
            c.setEnabled(false);
        (new Thread() {
            @Override
            public void run() {
                new UnpackManager(ThisFrame);
            }
        }).start();
    }
}
public void showFrame() {
    this.setLocationRelativeTo(null);
    this.setVisible(true);
}


private class UnpackProgressPanel extends JPanel {
    private JLabel UnpackLabel;
    private JProgressBar UnpackProgressBar;
    private JButton CancelButton;
    private int UnpackProgress = 0;
    public UnpackProgressPanel() {
        GridBagLayout gbl = new GridBagLayout();
        this.setLayout(gbl);

        GridBagConstraints gbc = new GridBagConstraints();

        //Adding label
        UnpackLabel = new JLabel("Unpacking: 0%");
        UnpackLabel.setFont(UnpackLabel.getFont().deriveFont(14f));
        this.add(UnpackLabel);
        gbc.weightx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.gridy++;
        gbc.insets = new Insets(5, 5, 5, 5);
        gbl.setConstraints(UnpackLabel, gbc);

        //Adding progress bar
        UnpackProgressBar = new JProgressBar();
        UnpackProgressBar.setMaximum(100);
        this.add(UnpackProgressBar);
        gbc.gridy++;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbl.setConstraints(UnpackProgressBar, gbc);

        //Adding cancel button
        CancelButton = new JButton("Cancel");
        CancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                CancelButton.setEnabled(false);
                UsedArchiveUnpackManager.pause();
            }
        });
        this.add(CancelButton);
        gbc.fill = GridBagConstraints.NONE;
        gbc.gridy++;
        gbc.anchor = GridBagConstraints.EAST;
        gbl.setConstraints(CancelButton, gbc);
    }

    public void reset() {
        UnpackProgress = 0;
        UnpackProgressBar.setValue(0);
        UnpackLabel.setText("Unpacking: 0%");
    }

    public void addOnePercent() {
        if(UnpackProgress < 100) {
            UnpackProgress++;
            UnpackProgressBar.setValue(UnpackProgress);
            UnpackLabel.setText("Unpacking: " + UnpackProgress+"%");
        }
    }

    public void setCancelButtonEnabled(boolean value) {
        CancelButton.setEnabled(value);
    }
}

//Probably a temporary solution
private class UnpackProgressDialog extends JDialog {
    private UnpackProgressPanel UsedProgressPanel;

    public UnpackProgressDialog(Frame owner) {
        super(owner);

        this.setLayout(new GridLayout(1, 1));
        UsedProgressPanel = new UnpackProgressPanel();
        this.add(UsedProgressPanel);
        UsedProgressPanel.setPreferredSize(new Dimension(500, 80));
        this.pack();
    }

    public void reset() {
        UsedProgressPanel.reset();
        this.setLocationRelativeTo(null);
    }

    public void showDialog() {
        this.setLocationRelativeTo(null);
        this.setVisible(true);
        this.toFront();
    }

    public void addOnePercent() {
        UsedProgressPanel.addOnePercent();
    }

     public void setCancelButtonEnabled(boolean value) {
         UsedProgressPanel.setCancelButtonEnabled(value);
     }
}

private class UnpackManager implements ArchiveUnpackManagerReportReceiver {
    private MainFrame ParentFrame;

    private UnpackManager(MainFrame f) {
        ParentFrame = f;
        UsedUPD.reset();
        UsedUPD.showDialog();
        try {
            UsedArchiveUnpackManager = new ArchiveUnpackManager(FileNameField.getText(), DestDirField.getText(), this);
            UsedArchiveUnpackManager.unpack();
        }
        catch(ArchiveUnpackManagerException e) {
            ErrorReportingDialog.showErrorMessage(ParentFrame, "Error", "An error happened while opening the archive!", e);
        }
    }

    private void resetWindow() {
        UsedUPD.setVisible(false);
        UsedUPD.setCancelButtonEnabled(true);
        for(JComponent c : ComponentsToDisable) c.setEnabled(true);
    }

    public void report(int Report) {
        if(Report == UNPACK_SUCCESS) {
            JOptionPane.showMessageDialog(ParentFrame, "Unpacking finished", "Success", JOptionPane.INFORMATION_MESSAGE);
            resetWindow();
        }
        else if(Report == UNPACK_FAILED) {
            JOptionPane.showMessageDialog(ParentFrame, "An error happened during unpacking", "Error", JOptionPane.ERROR_MESSAGE);
            resetWindow();
        }
        else if(Report == ONE_PERCENT_UNPACKED) {
            UsedUPD.addOnePercent();
        }
    }

    public void report(int Report, String FilePath) {
        JOptionPane.showMessageDialog(ParentFrame, "A file was unpacked but it is of unexpected size. It may be corrupted. Path: "+FilePath, "Possible error", JOptionPane.WARNING_MESSAGE);
    }

    public void report(int Report, Exception e) {
        if(e instanceof UnpackException) {
            if(((UnpackException)e).getReason() == UnpackException.REASON_OPENNEXTSOURCE_FAILED) {
                JOptionPane.showMessageDialog(ParentFrame, "Cannot open file " +
                        UsedArchiveUnpackManager.getCurrentSourceInputFile().getName(),
                        "Error", JOptionPane.ERROR_MESSAGE);
                resetWindow();
                return;
            }
        }
        if(Report == UNPACK_FAILED) {
            ErrorReportingDialog.showErrorMessage(ParentFrame, "Error", "An error happened while unpacking. Below are more details.", e);
            resetWindow();
        }
    }

    public boolean verifyAbort() {
        if(JOptionPane.showConfirmDialog(ParentFrame,"Are you sure you want to abort unpacking?", "Abort", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
            return true;
        }
        else {
            return false;
        }
    }

    public void reportTMFPUnpacked() {
        JOptionPane.showMessageDialog(ThisFrame, "<html>This is an uncompressed TMFP archive." +
                "We cannot check its integrity yet.<br>If you think this archive could be corrupted (e.g. while downloading), please check.<br>" +
                "If it contains another archive, e.g. RAR, don't worry.</html>", "Warning", JOptionPane.WARNING_MESSAGE);
    }

    public void reportUnpackingAbortedByUser() {
        resetWindow();
    }
}
}
